/*
 * Plugin example for XnView 1.80 or more
 *
 * When Xuser is built, put it in Plugins folder
 */

#include <stdio.h>
#include <windows.h>

BOOL APIENTRY DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved) 
{
	switch (ul_reason_for_call) 
	{
	case DLL_PROCESS_ATTACH :
	case DLL_PROCESS_DETACH :
	case DLL_THREAD_ATTACH  :
	case DLL_THREAD_DETACH  :
		break;
  }

	return TRUE;
}

#define API __stdcall

#define GFP_RGB		0
#define GFP_BGR		1
#define GFP_GREY	5

#define GFP_READ	0x0001
#define GFP_WRITE 0x0002

typedef struct {
		unsigned char red[256]; 
		unsigned char green[256]; 
		unsigned char blue[256]; 
	} GFP_COLORMAP; 

BOOL API gfpGetPluginInfo( DWORD version, LPSTR label, INT label_max_size, LPSTR extension, INT extension_max_size, INT * support )
{
	if ( version != 0x0002 )
		return FALSE; 

	strncpy( label, "Plugin Example", label_max_size ); 
	strcpy( extension, "tst;test" ); 

	*support = GFP_READ | GFP_WRITE; 

	return TRUE; 
}

typedef struct {
		FILE * fp; 
		INT width; 
		INT component_per_pixel; 
	} MYDATA; 

void * API gfpLoadPictureInit( LPCSTR filename )
{
	MYDATA * data; 

	data = calloc( 1, sizeof(MYDATA) ); 

	data->fp = fopen( filename, "rb" ); 
	if ( data->fp == NULL )
	{
		free(data); 
		data = NULL; 
	}

	return data; 
}

BOOL API gfpLoadPictureGetInfo( void * ptr, INT * pictype, INT * width, INT * height, INT * dpi, INT * bits_per_pixel, INT * bytes_per_line, BOOL * has_colormap, LPSTR label, INT label_max_size )
{
	MYDATA * data = (MYDATA *)ptr; 

	fread( width, sizeof(INT), 1, data->fp ); 
	fread( height, sizeof(INT), 1, data->fp ); 
	fread( bits_per_pixel, sizeof(INT), 1, data->fp ); 

	data->width = *width; 
	data->component_per_pixel = *bits_per_pixel / 8; 

	*pictype = GFP_RGB; 
	*dpi = 68; 
	*bytes_per_line = *bits_per_pixel/8 * *width; 
	
	*has_colormap = FALSE; 
	
	strncpy( label, "Test format", label_max_size ); 

	return TRUE; 
}

BOOL API gfpLoadPictureGetLine( void * ptr, INT line, unsigned char * buffer )
{
	MYDATA * data = (MYDATA *)ptr; 

	fread( buffer, data->width, data->component_per_pixel, data->fp ); 
	return TRUE; 
}

BOOL API gfpLoadPictureGetColormap( void * ptr, GFP_COLORMAP * cmap )
{
	return FALSE; 
}

void API gfpLoadPictureExit( void * ptr )
{
	MYDATA * data = (MYDATA *)ptr; 

	fclose(data->fp); 
	free(data); 
}

// bits_per_pixel can be 1 to 8, 24, 32
BOOL API gfpSavePictureIsSupported( INT width, INT height, INT bits_per_pixel, BOOL has_colormap )
{
	if ( has_colormap
		|| bits_per_pixel < 8 )
		return FALSE; 

	return TRUE; 
}

void * API gfpSavePictureInit( LPCSTR filename, INT width, INT height, INT bits_per_pixel, INT dpi, INT * picture_type, LPSTR label, INT label_max_size )
{
	MYDATA * data; 

	data = calloc( 1, sizeof(MYDATA) ); 

	data->fp = fopen( filename, "wb" ); 
	if ( data->fp == NULL )
	{
		free(data); 
		data = NULL; 
	}

	*picture_type = GFP_RGB; 

	fwrite( &width, sizeof(width), 1, data->fp ); 
	fwrite( &height, sizeof(height), 1, data->fp ); 
	fwrite( &bits_per_pixel, sizeof(bits_per_pixel), 1, data->fp ); 

	strncpy( label, "Test format", label_max_size ); 

	data->width = width; 
	data->component_per_pixel = bits_per_pixel / 8; 
	return data; 
}

BOOL API gfpSavePicturePutLine( void * ptr, INT line, const unsigned char * buffer )
{
	MYDATA * data = (MYDATA *)ptr; 

	fwrite( buffer, data->width, data->component_per_pixel, data->fp ); 
	return TRUE; 
}

void API gfpSavePictureExit( void * ptr )
{
	MYDATA * data = (MYDATA *)ptr; 

	fclose(data->fp); 
	free(data); 
}

